@@ -1,7 +1,8 @@
 import re
+import urllib.parse as urlparse
 
 from django.conf import settings
-from django.core.paginator import Paginator
+from django.core.paginator import InvalidPage, Paginator
 from django.http import Http404
 from django.shortcuts import get_object_or_404
 from django.template.response import TemplateResponse
@@ -21,7 +22,7 @@ from wagtail.admin.forms.search import SearchForm
 from wagtail.admin.modal_workflow import render_modal_workflow
 from wagtail.admin.ui.tables import Column, DateColumn, Table
 from wagtail.coreutils import resolve_model_string
-from wagtail.models import Locale, Page, Site, UserPagePermissionsProxy
+from wagtail.models import Locale, Page, Site
 
 
 def shared_context(request, extra_context=None):
@@ -57,7 +58,7 @@ def page_models_from_string(string):
 
 def can_choose_page(
     page,
-    permission_proxy,
+    user,
     desired_classes,
     can_choose_root=True,
     user_perm=None,
@@ -91,9 +92,9 @@ def can_choose_page(
                 return False
 
             if user_perm == "move_to":
-                return permission_proxy.for_page(page_to_move).can_move_to(page)
+                return page_to_move.permissions_for_user(user).can_move_to(page)
     if user_perm == "copy_to":
-        return permission_proxy.for_page(page).can_add_subpage()
+        return page.permissions_for_user(user).can_add_subpage()
 
     return True
 
@@ -101,6 +102,15 @@ def can_choose_page(
 class PageChooserTable(Table):
     classname = "listing chooser"
 
+    def __init__(self, *args, show_locale_labels=False, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.show_locale_labels = show_locale_labels
+
+    def get_context_data(self, parent_context):
+        context = super().get_context_data(parent_context)
+        context["show_locale_labels"] = self.show_locale_labels
+        return context
+
     def get_row_classname(self, page):
         classnames = []
         if page.is_parent_page:
@@ -116,9 +126,9 @@ class PageChooserTable(Table):
 class PageTitleColumn(Column):
     cell_template_name = "wagtailadmin/chooser/tables/page_title_cell.html"
 
-    def __init__(self, *args, show_locale_labels=False, **kwargs):
+    def __init__(self, *args, is_multiple_choice=False, **kwargs):
         super().__init__(*args, **kwargs)
-        self.show_locale_labels = show_locale_labels
+        self.is_multiple_choice = is_multiple_choice
 
     def get_value(self, instance):
         return instance.get_admin_display_title()
@@ -126,19 +136,24 @@ class PageTitleColumn(Column):
     def get_cell_context_data(self, instance, parent_context):
         context = super().get_cell_context_data(instance, parent_context)
         context["page"] = instance
+        # only need to show locale labels for top-level pages
+        context["show_locale_labels"] = (
+            parent_context.get("show_locale_labels") and instance.depth == 2
+        )
         return context
 
 
 class ParentPageColumn(Column):
     cell_template_name = "wagtailadmin/chooser/tables/parent_page_cell.html"
 
-    def __init__(self, *args, show_locale_labels=False, **kwargs):
-        super().__init__(*args, **kwargs)
-        self.show_locale_labels = show_locale_labels
-
     def get_value(self, instance):
         return instance.get_parent()
 
+    def get_cell_context_data(self, instance, parent_context):
+        context = super().get_cell_context_data(instance, parent_context)
+        context["show_locale_labels"] = parent_context.get("show_locale_labels")
+        return context
+
 
 class PageStatusColumn(Column):
     cell_template_name = "wagtailadmin/chooser/tables/page_status_cell.html"
@@ -156,12 +171,18 @@ class PageNavigateToChildrenColumn(Column):
         return instance
 
 
+class PageCheckboxSelectColumn(Column):
+    cell_template_name = "wagtailadmin/chooser/tables/page_checkbox_select_cell.html"
+
+
 class BrowseView(View):
     @property
     def columns(self):
-        return [
+        cols = [
             PageTitleColumn(
-                "title", label=_("Title"), show_locale_labels=self.i18n_enabled
+                "title",
+                label=_("Title"),
+                is_multiple_choice=self.is_multiple_choice,
             ),
             DateColumn(
                 "updated",
@@ -178,6 +199,14 @@ class BrowseView(View):
             PageStatusColumn("status", label=_("Status"), width="12%"),
             PageNavigateToChildrenColumn("children", label="", width="10%"),
         ]
+        if self.is_multiple_choice:
+            cols.insert(
+                0,
+                PageCheckboxSelectColumn(
+                    "select", label=_("Select"), width="1%", accessor="pk"
+                ),
+            )
+        return cols
 
     def get_object_list(self):
         # Get children of parent page (without streamfields)
@@ -204,6 +233,7 @@ class BrowseView(View):
 
     def get(self, request, parent_page_id=None):
         self.i18n_enabled = getattr(settings, "WAGTAIL_I18N_ENABLED", False)
+        self.is_multiple_choice = request.GET.get("multiple")
 
         # A missing or empty page_type parameter indicates 'all page types'
         # (i.e. descendants of wagtailcore.page)
@@ -240,13 +270,10 @@ class BrowseView(View):
 
         match_subclass = request.GET.get("match_subclass", True)
 
-        # Do permission lookups for this user now, instead of for every page.
-        permission_proxy = UserPagePermissionsProxy(request.user)
-
         # Parent page can be chosen if it is a instance of desired_classes
         self.parent_page.can_choose = can_choose_page(
             self.parent_page,
-            permission_proxy,
+            request.user,
             self.desired_classes,
             can_choose_root,
             user_perm,
@@ -323,13 +350,16 @@ class BrowseView(View):
         # We apply pagination first so we don't need to walk the entire list
         # in the block below
         paginator = Paginator(pages, per_page=25)
-        pages = paginator.get_page(request.GET.get("p"))
+        try:
+            pages = paginator.page(request.GET.get("p", 1))
+        except InvalidPage:
+            raise Http404
 
         # Annotate each page with can_choose/can_decend flags
         for page in pages:
             page.can_choose = can_choose_page(
                 page,
-                permission_proxy,
+                request.user,
                 self.desired_classes,
                 can_choose_root,
                 user_perm,
@@ -342,6 +372,7 @@ class BrowseView(View):
         table = PageChooserTable(
             self.columns,
             [self.parent_page] + list(pages),
+            show_locale_labels=self.i18n_enabled,
         )
 
         # Render
@@ -359,9 +390,10 @@ class BrowseView(View):
                     for desired_class in self.desired_classes
                 ],
                 "page_types_restricted": (page_type_string != "wagtailcore.page"),
-                "show_locale_labels": self.i18n_enabled,
+                "show_locale_controls": self.i18n_enabled,
                 "locale_options": locale_options,
                 "selected_locale": selected_locale,
+                "is_multiple_choice": self.is_multiple_choice,
             },
         )
 
@@ -377,13 +409,9 @@ class BrowseView(View):
 class SearchView(View):
     @property
     def columns(self):
-        return [
-            PageTitleColumn(
-                "title", label=_("Title"), show_locale_labels=self.i18n_enabled
-            ),
-            ParentPageColumn(
-                "parent", label=_("Parent"), show_locale_labels=self.i18n_enabled
-            ),
+        cols = [
+            PageTitleColumn("title", label=_("Title")),
+            ParentPageColumn("parent", label=_("Parent")),
             DateColumn(
                 "updated",
                 label=_("Updated"),
@@ -398,9 +426,18 @@ class SearchView(View):
             ),
             PageStatusColumn("status", label=_("Status"), width="12%"),
         ]
+        if self.is_multiple_choice:
+            cols.insert(
+                0,
+                PageCheckboxSelectColumn(
+                    "select", label=_("Select"), width="1%", accessor="pk"
+                ),
+            )
+        return cols
 
     def get(self, request):
         self.i18n_enabled = getattr(settings, "WAGTAIL_I18N_ENABLED", False)
+        self.is_multiple_choice = request.GET.get("multiple")
         # A missing or empty page_type parameter indicates 'all page types' (i.e. descendants of wagtailcore.page)
         page_type_string = request.GET.get("page_type") or "wagtailcore.page"
 
@@ -422,7 +459,8 @@ class SearchView(View):
             pages = pages.exclude(depth=1)  # never include root
             pages = pages.type(*desired_classes)
             pages = pages.specific()
-            pages = pages.search(search_form.cleaned_data["q"])
+            pages = pages.autocomplete(search_form.cleaned_data["q"])
+
         else:
             pages = pages.none()
 
@@ -436,6 +474,7 @@ class SearchView(View):
         table = PageChooserTable(
             self.columns,
             pages,
+            show_locale_labels=self.i18n_enabled,
         )
 
         return TemplateResponse(
@@ -448,12 +487,42 @@ class SearchView(View):
                     "table": table,
                     "pages": pages,
                     "page_type_string": page_type_string,
-                    "show_locale_labels": self.i18n_enabled,
                 },
             ),
         )
 
 
+class ChosenMultipleView(View):
+    """
+    A view that takes a list of 'id' URL parameters and returns a modal workflow response indicating
+    that those objects have been chosen
+    """
+
+    def render_chosen_response(self, result):
+        return render_modal_workflow(
+            self.request,
+            None,
+            None,
+            None,
+            json_data={"step": "page_chosen", "result": result},
+        )
+
+    def get(self, request):
+        pks = request.GET.getlist("id")
+        pages = Page.objects.filter(pk__in=pks).specific()
+        result = [
+            {
+                "id": page.pk,
+                "parentId": page.get_parent().pk,
+                "adminTitle": page.get_admin_display_title(),
+                "editUrl": reverse("wagtailadmin_pages:edit", args=(page.pk,)),
+                "url": page.url,
+            }
+            for page in pages
+        ]
+        return self.render_chosen_response(result)
+
+
 class BaseLinkFormView(View):
     def get_initial_data(self):
         return {
@@ -666,9 +735,66 @@ class EmailLinkView(BaseLinkFormView):
     step_name = "email_link"
     link_url_field_name = "email_address"
 
+    def get_initial_data(self):
+        parsed_email = self.parse_email_link(self.request.GET.get("link_url", ""))
+        return {
+            "email_address": parsed_email["email"],
+            "link_text": self.request.GET.get("link_text", ""),
+            "subject": parsed_email["subject"],
+            "body": parsed_email["body"],
+        }
+
     def get_url_from_field_value(self, value):
         return "mailto:" + value
 
+    def get_result_data(self):
+        params = {
+            "subject": self.form.cleaned_data["subject"],
+            "body": self.form.cleaned_data["body"],
+        }
+        encoded_params = urlparse.urlencode(
+            {k: v for k, v in params.items() if v is not None and v != ""},
+            quote_via=urlparse.quote,
+        )
+
+        url = "mailto:" + self.form.cleaned_data["email_address"]
+        if encoded_params:
+            url += "?" + encoded_params
+
+        return {
+            "url": url,
+            "title": self.form.cleaned_data["link_text"].strip()
+            or self.form.cleaned_data["email_address"],
+            # If the user has explicitly entered / edited something in the link_text field,
+            # always use that text. If not, we should favour keeping the existing link/selection
+            # text, where applicable.
+            "prefer_this_title_as_link_text": ("link_text" in self.form.changed_data),
+        }
+
+    def post(self, request):
+        self.form = self.form_class(
+            request.POST, initial=self.get_initial_data(), prefix=self.form_prefix
+        )
+
+        if self.form.is_valid():
+            result = self.get_result_data()
+            return self.render_chosen_response(result)
+        else:  # form invalid
+            return self.render_form_response()
+
+    def parse_email_link(self, mailto):
+        result = {}
+
+        mail_result = urlparse.urlparse(mailto)
+
+        result["email"] = mail_result.path
+
+        query = urlparse.parse_qs(mail_result.query)
+        result["subject"] = query["subject"][0] if "subject" in query else ""
+        result["body"] = query["body"][0] if "body" in query else ""
+
+        return result
+
 
 class PhoneLinkView(BaseLinkFormView):
     form_prefix = "phone-link-chooser"
@@ -678,4 +804,5 @@ class PhoneLinkView(BaseLinkFormView):
     link_url_field_name = "phone_number"
 
     def get_url_from_field_value(self, value):
+        value = re.sub(r"\s", "", value)
         return "tel:" + value
