@@ -2,9 +2,11 @@ import itertools
 import uuid
 from collections import OrderedDict, defaultdict
 from collections.abc import Mapping, MutableSequence
+from pickle import PickleError
 
 from django import forms
-from django.core.exceptions import NON_FIELD_ERRORS, ValidationError
+from django.core.exceptions import ValidationError
+from django.db.models.fields import _load_field
 from django.forms.utils import ErrorList
 from django.utils.functional import cached_property
 from django.utils.html import format_html_join
@@ -13,7 +15,14 @@ from django.utils.translation import gettext as _
 from wagtail.admin.staticfiles import versioned_static
 from wagtail.telepath import Adapter, register
 
-from .base import Block, BoundBlock, DeclarativeSubBlocksMetaclass, get_help_icon
+from .base import (
+    Block,
+    BoundBlock,
+    DeclarativeSubBlocksMetaclass,
+    get_error_json_data,
+    get_error_list_json_data,
+    get_help_icon,
+)
 
 __all__ = [
     "BaseStreamBlock",
@@ -25,44 +34,50 @@ __all__ = [
 
 class StreamBlockValidationError(ValidationError):
     def __init__(self, block_errors=None, non_block_errors=None):
-        self.non_block_errors = non_block_errors
-        self.block_errors = block_errors
+        # non_block_errors may be passed here as an ErrorList, a plain list (of strings or
+        # ValidationErrors), or None.
+        # Normalise it to be an ErrorList, which provides an as_data() method that consistently
+        # returns a flat list of ValidationError objects.
+        self.non_block_errors = ErrorList(non_block_errors)
 
-        params = {}
-        if block_errors:
-            params.update(block_errors)
-        if non_block_errors:
-            params[NON_FIELD_ERRORS] = non_block_errors
-        super().__init__("Validation error in StreamBlock", params=params)
+        # block_errors may be passed here as None, or a dict keyed by the indexes of the child blocks
+        # with errors.
+        # Items in this list / dict may be:
+        #  - a ValidationError instance (potentially a subclass such as StructBlockValidationError)
+        #  - an ErrorList containing a single ValidationError
+        #  - a plain list containing a single ValidationError
+        # All representations will be normalised to a dict of ValidationError instances,
+        # which is also the preferred format for the original argument to be in.
+        self.block_errors = {}
+        if block_errors is None:
+            pass
+        else:
+            for index, val in block_errors.items():
+                if isinstance(val, ErrorList):
+                    self.block_errors[index] = val.as_data()[0]
+                elif isinstance(val, list):
+                    self.block_errors[index] = val[0]
+                else:
+                    self.block_errors[index] = val
 
+        super().__init__("Validation error in StreamBlock")
 
-class StreamBlockValidationErrorAdapter(Adapter):
-    js_constructor = "wagtail.blocks.StreamBlockValidationError"
-
-    def js_args(self, error):
-        return [
-            error.non_block_errors.as_data(),
-            {
-                block_id: child_errors.as_data()
-                for block_id, child_errors in error.block_errors.items()
-            },
-        ]
-
-    @cached_property
-    def media(self):
-        return forms.Media(
-            js=[
-                versioned_static("wagtailadmin/js/telepath/blocks.js"),
-            ]
-        )
-
-
-register(StreamBlockValidationErrorAdapter(), StreamBlockValidationError)
+    def as_json_data(self):
+        result = {}
+        if self.non_block_errors:
+            result["messages"] = get_error_list_json_data(self.non_block_errors)
+        if self.block_errors:
+            result["blockErrors"] = {
+                index: get_error_json_data(error)
+                for (index, error) in self.block_errors.items()
+            }
+        return result
 
 
 class BaseStreamBlock(Block):
-    def __init__(self, local_blocks=None, **kwargs):
+    def __init__(self, local_blocks=None, search_index=True, **kwargs):
         self._constructor_kwargs = kwargs
+        self.search_index = search_index
 
         super().__init__(**kwargs)
 
@@ -154,12 +169,13 @@ class BaseStreamBlock(Block):
                     (child.block.name, child.block.clean(child.value), child.id)
                 )
             except ValidationError as e:
-                errors[i] = ErrorList([e])
+                errors[i] = e
 
         if self.meta.min_num is not None and self.meta.min_num > len(value):
             non_block_errors.append(
                 ValidationError(
-                    _("The minimum number of items is %d") % self.meta.min_num
+                    _("The minimum number of items is %(min_num)d")
+                    % {"min_num": self.meta.min_num}
                 )
             )
         elif self.required and len(value) == 0:
@@ -168,7 +184,8 @@ class BaseStreamBlock(Block):
         if self.meta.max_num is not None and self.meta.max_num < len(value):
             non_block_errors.append(
                 ValidationError(
-                    _("The maximum number of items is %d") % self.meta.max_num
+                    _("The maximum number of items is %(max_num)d")
+                    % {"max_num": self.meta.max_num}
                 )
             )
 
@@ -187,7 +204,8 @@ class BaseStreamBlock(Block):
                         ValidationError(
                             "{}: {}".format(
                                 block.label,
-                                _("The minimum number of items is %d") % min_num,
+                                _("The minimum number of items is %(min_num)d")
+                                % {"min_num": min_num},
                             )
                         )
                     )
@@ -196,7 +214,8 @@ class BaseStreamBlock(Block):
                         ValidationError(
                             "{}: {}".format(
                                 block.label,
-                                _("The maximum number of items is %d") % max_num,
+                                _("The maximum number of items is %(max_num)d")
+                                % {"max_num": max_num},
                             )
                         )
                     )
@@ -322,6 +341,8 @@ class BaseStreamBlock(Block):
         )
 
     def get_searchable_content(self, value):
+        if not self.search_index:
+            return []
         content = []
 
         for child in value:
@@ -329,6 +350,40 @@ class BaseStreamBlock(Block):
 
         return content
 
+    def extract_references(self, value):
+        for child in value:
+            for (
+                model,
+                object_id,
+                model_path,
+                content_path,
+            ) in child.block.extract_references(child.value):
+                model_path = (
+                    f"{child.block_type}.{model_path}"
+                    if model_path
+                    else child.block_type
+                )
+                content_path = (
+                    f"{child.id}.{content_path}" if content_path else child.id
+                )
+                yield model, object_id, model_path, content_path
+
+    def get_block_by_content_path(self, value, path_elements):
+        """
+        Given a list of elements from a content path, retrieve the block at that path
+        as a BoundBlock object, or None if the path does not correspond to a valid block.
+        """
+        if path_elements:
+            id, *remaining_elements = path_elements
+            for child in value:
+                if child.id == id:
+                    return child.block.get_block_by_content_path(
+                        child.value, remaining_elements
+                    )
+        else:
+            # an empty path refers to the stream as a whole
+            return self.bind(value)
+
     def deconstruct(self):
         """
         Always deconstruct StreamBlock instances as if they were plain StreamBlocks with all of the
@@ -395,7 +450,7 @@ class StreamValue(MutableSequence):
 
         def __init__(self, *args, **kwargs):
             self.id = kwargs.pop("id")
-            super(StreamValue.StreamChild, self).__init__(*args, **kwargs)
+            super().__init__(*args, **kwargs)
 
         @property
         def block_type(self):
@@ -500,8 +555,7 @@ class StreamValue(MutableSequence):
             return result
 
         def __iter__(self):
-            for block_name in self.block_names:
-                yield block_name
+            yield from self.block_names
 
         def __len__(self):
             return len(self.block_names)
@@ -676,7 +730,7 @@ class StreamValue(MutableSequence):
         return len(self._bound_blocks)
 
     def __repr__(self):
-        return "<%s %r>" % (type(self).__name__, list(self))
+        return f"<{type(self).__name__} {list(self)!r}>"
 
     def render_as_block(self, context=None):
         return self.stream_block.render(self, context=context)
@@ -687,6 +741,30 @@ class StreamValue(MutableSequence):
     def __str__(self):
         return self.__html__()
 
+    @staticmethod
+    def _deserialize_pickle_value(app_label, model_name, field_name, field_value):
+        """Returns StreamValue from pickled data"""
+        field = _load_field(app_label, model_name, field_name)
+        return field.to_python(field_value)
+
+    def __reduce__(self):
+        try:
+            stream_field = self._stream_field
+        except AttributeError:
+            raise PickleError(
+                "StreamValue can only be pickled if it is associated with a StreamField"
+            )
+
+        return (
+            self._deserialize_pickle_value,
+            (
+                stream_field.model._meta.app_label,
+                stream_field.model._meta.object_name,
+                stream_field.name,
+                self.get_prep_value(),
+            ),
+        )
+
 
 class StreamBlockAdapter(Adapter):
     js_constructor = "wagtail.blocks.StreamBlock"
