@@ -1,8 +1,8 @@
 import os
 
-import tablib
 from django.core.management.base import BaseCommand
 
+from wagtail.contrib.redirects.base_formats import Dataset
 from wagtail.contrib.redirects.forms import RedirectForm
 from wagtail.contrib.redirects.utils import (
     get_format_cls_by_extension,
@@ -12,7 +12,7 @@ from wagtail.models import Site
 
 
 class Command(BaseCommand):
-    help = "Imports redirects from .csv, .xls, .xlsx"
+    help = "Imports redirects from a .csv, .tsv or .xlsx file"
 
     def add_arguments(self, parser):
         parser.add_argument(
@@ -61,7 +61,7 @@ class Command(BaseCommand):
         )
         parser.add_argument(
             "--format",
-            help="Source file format (example: .csv, .xls etc)",
+            help="Source file format (csv, tsv or xlsx)",
             choices=get_supported_extensions(),
             type=str,
         )
@@ -95,10 +95,10 @@ class Command(BaseCommand):
             site = Site.objects.get(id=site_id)
 
         if not os.path.exists(src):
-            raise Exception("Missing file '{0}'".format(src))
+            raise Exception(f"Missing file '{src}'")
 
         if not os.path.getsize(src) > 0:
-            raise Exception("File '{0}' is empty".format(src))
+            raise Exception(f"File '{src}' is empty")
 
         _, extension = os.path.splitext(src)
         extension = extension.lstrip(".")
@@ -106,8 +106,10 @@ class Command(BaseCommand):
         if not format_:
             format_ = extension
 
-        if not get_format_cls_by_extension(format_):
-            raise Exception("Invalid format '{0}'".format(extension))
+        import_format_cls = get_format_cls_by_extension(format_)
+        if import_format_cls is None:
+            raise Exception(f"Invalid format '{extension}'")
+        input_format = import_format_cls()
 
         if extension in ["xls", "xlsx"]:
             mode = "rb"
@@ -115,23 +117,16 @@ class Command(BaseCommand):
             mode = "r"
 
         with open(src, mode) as fh:
-            imported_data = tablib.Dataset().load(fh.read(), format=format_)
+            imported_data = input_format.create_dataset(fh.read())
+            sample_data = Dataset(imported_data[:4], imported_data.headers)
 
-            sample_data = tablib.Dataset(
-                *imported_data[: min(len(imported_data), 4)],
-                headers=imported_data.headers,
-            )
-
-            try:
-                self.stdout.write("Sample data:")
-                self.stdout.write(str(sample_data))
-            except Exception:
-                self.stdout.write("Warning: Cannot display sample data")
+            self.stdout.write("Sample data:")
+            self.stdout.write(str(sample_data))
 
             self.stdout.write("--------------")
 
             if site:
-                self.stdout.write("Using site: {0}".format(site.hostname))
+                self.stdout.write(f"Using site: {site.hostname}")
 
             self.stdout.write("Importing redirects:")
 
@@ -198,10 +193,10 @@ class Command(BaseCommand):
                 successes += 1
 
         self.stdout.write("\n")
-        self.stdout.write("Found: {}".format(total))
-        self.stdout.write("Created: {}".format(successes))
-        self.stdout.write("Skipped : {}".format(skipped))
-        self.stdout.write("Errors: {}".format(len(errors)))
+        self.stdout.write(f"Found: {total}")
+        self.stdout.write(f"Created: {successes}")
+        self.stdout.write(f"Skipped : {skipped}")
+        self.stdout.write(f"Errors: {len(errors)}")
 
 
 def get_input(msg):  # pragma: no cover
