@@ -2,22 +2,21 @@ import os
 
 from django.conf import settings
 from django.core.exceptions import PermissionDenied
-from django.core.paginator import Paginator
 from django.http import HttpResponse, JsonResponse
 from django.shortcuts import get_object_or_404, redirect
 from django.template.response import TemplateResponse
 from django.urls import reverse
 from django.urls.exceptions import NoReverseMatch
-from django.utils.decorators import method_decorator
 from django.utils.http import urlencode
 from django.utils.translation import gettext as _
-from django.views.generic import TemplateView
+from django.utils.translation import gettext_lazy, ngettext
+from django.views import View
 
 from wagtail.admin import messages
 from wagtail.admin.auth import PermissionPolicyChecker
-from wagtail.admin.forms.search import SearchForm
 from wagtail.admin.models import popular_tags_for_model
-from wagtail.admin.views.pages.utils import get_valid_next_url_from_request
+from wagtail.admin.utils import get_valid_next_url_from_request, set_query_params
+from wagtail.admin.views import generic
 from wagtail.images import get_image_model
 from wagtail.images.exceptions import InvalidFilterSpecError
 from wagtail.images.forms import URLGeneratorForm, get_image_form
@@ -32,7 +31,7 @@ INDEX_PAGE_SIZE = getattr(settings, "WAGTAILIMAGES_INDEX_PAGE_SIZE", 30)
 USAGE_PAGE_SIZE = getattr(settings, "WAGTAILIMAGES_USAGE_PAGE_SIZE", 20)
 
 
-class BaseListingView(TemplateView):
+class IndexView(generic.IndexView):
     ENTRIES_PER_PAGE_CHOICES = sorted({10, 30, 60, 100, 250, INDEX_PAGE_SIZE})
     ORDERING_OPTIONS = {
         "-created_at": _("Newest"),
@@ -43,12 +42,22 @@ class BaseListingView(TemplateView):
         "-file_size": _("File size: (high to low)"),
     }
     default_ordering = "-created_at"
+    context_object_name = "images"
+    permission_policy = permission_policy
+    any_permission_required = ["add", "change", "delete"]
+    model = get_image_model()
+    show_other_searches = True
+    header_icon = "image"
+    page_title = gettext_lazy("Images")
+    add_item_label = gettext_lazy("Add an image")
+    index_url_name = "wagtailimages:index"
+    index_results_url_name = "wagtailimages:index_results"
+    add_url_name = "wagtailimages:add_multiple"
+    edit_url_name = "wagtailimages:edit"
+    template_name = "wagtailimages/images/index.html"
+    results_template_name = "wagtailimages/images/index_results.html"
 
-    @method_decorator(permission_checker.require_any("add", "change", "delete"))
-    def get(self, request):
-        return super().get(request)
-
-    def get_num_entries_per_page(self):
+    def get_paginate_by(self, queryset):
         entries_per_page = self.request.GET.get("entries_per_page", INDEX_PAGE_SIZE)
         try:
             entries_per_page = int(entries_per_page)
@@ -57,124 +66,94 @@ class BaseListingView(TemplateView):
         if entries_per_page not in self.ENTRIES_PER_PAGE_CHOICES:
             entries_per_page = INDEX_PAGE_SIZE
 
+        self.entries_per_page = entries_per_page
+
         return entries_per_page
 
     def get_valid_orderings(self):
         return self.ORDERING_OPTIONS
 
-    def get_ordering(self):
-        # TODO: remove this method when this view will be based on the
-        # generic model index view from wagtail.admin.views.generic.models.IndexView
-        ordering = self.request.GET.get("ordering")
-        if ordering is None or ordering not in self.get_valid_orderings():
-            ordering = self.default_ordering
-        return ordering
-
-    def get_context_data(self, **kwargs):
-        context = super().get_context_data(**kwargs)
-
-        # Get ordering
-        ordering = self.get_ordering()
-
-        # Get images (filtered by user permission and ordered by `ordering`)
+    def get_base_queryset(self):
+        # Get images (filtered by user permission)
         images = (
             permission_policy.instances_user_has_any_permission_for(
                 self.request.user, ["change", "delete"]
             )
-            .order_by(ordering)
             .select_related("collection")
             .prefetch_renditions("max-165x165")
         )
+        return images
 
+    def filter_queryset(self, queryset):
         # Filter by collection
         self.current_collection = None
         collection_id = self.request.GET.get("collection_id")
         if collection_id:
             try:
                 self.current_collection = Collection.objects.get(id=collection_id)
-                images = images.filter(collection=self.current_collection)
+                queryset = queryset.filter(collection=self.current_collection)
             except (ValueError, Collection.DoesNotExist):
                 pass
 
-        # Search
-        query_string = None
-        if "q" in self.request.GET:
-            self.form = SearchForm(self.request.GET, placeholder=_("Search images"))
-            if self.form.is_valid():
-                query_string = self.form.cleaned_data["q"]
-
-                images = images.search(query_string)
-        else:
-            self.form = SearchForm(placeholder=_("Search images"))
-
         # Filter by tag
         self.current_tag = self.request.GET.get("tag")
-        if self.current_tag:
+        # Combining search with tag filter is not yet supported, see
+        # https://github.com/wagtail/wagtail/issues/6616
+        if self.current_tag and not self.search_query:
             try:
-                images = images.filter(tags__name=self.current_tag)
-            except (AttributeError):
+                queryset = queryset.filter(tags__name=self.current_tag)
+            except AttributeError:
                 self.current_tag = None
 
-        entries_per_page = self.get_num_entries_per_page()
-        paginator = Paginator(images, per_page=entries_per_page)
-        images = paginator.get_page(self.request.GET.get("p"))
+        return queryset
 
-        next_url = reverse("wagtailimages:index")
+    def get_add_url(self):
+        # Pass the query string so that the collection filter is preserved
+        return set_query_params(
+            super().get_add_url(),
+            self.request.GET.copy(),
+        )
+
+    def get_next_url(self):
+        next_url = self.index_url
         request_query_string = self.request.META.get("QUERY_STRING")
         if request_query_string:
             next_url += "?" + request_query_string
-
-        context.update(
-            {
-                "images": images,
-                "query_string": query_string,
-                "is_searching": bool(query_string),
-                "next": next_url,
-                "entries_per_page": entries_per_page,
-                "ENTRIES_PER_PAGE_CHOICES": self.ENTRIES_PER_PAGE_CHOICES,
-                "current_ordering": ordering,
-                "ORDERING_OPTIONS": self.ORDERING_OPTIONS,
-            }
-        )
-
-        return context
-
-
-class IndexView(BaseListingView):
-    template_name = "wagtailimages/images/index.html"
+        return next_url
 
     def get_context_data(self, **kwargs):
         context = super().get_context_data(**kwargs)
 
-        collections = permission_policy.collections_user_has_any_permission_for(
+        context.update(
+            {
+                "next": self.get_next_url(),
+                "entries_per_page": self.entries_per_page,
+                "current_tag": self.current_tag,
+                "current_collection": self.current_collection,
+                "ENTRIES_PER_PAGE_CHOICES": self.ENTRIES_PER_PAGE_CHOICES,
+                "current_ordering": self.ordering,
+                "ORDERING_OPTIONS": self.ORDERING_OPTIONS,
+            }
+        )
+
+        if self.results_only:
+            return context
+
+        collections = self.permission_policy.collections_user_has_any_permission_for(
             self.request.user, ["add", "change"]
         )
         if len(collections) < 2:
             collections = None
 
-        Image = get_image_model()
-
         context.update(
             {
-                "search_form": self.form,
                 "popular_tags": popular_tags_for_model(get_image_model()),
-                "current_tag": self.current_tag,
                 "collections": collections,
-                "current_collection": self.current_collection,
-                "user_can_add": permission_policy.user_has_permission(
-                    self.request.user, "add"
-                ),
-                "app_label": Image._meta.app_label,
-                "model_name": Image._meta.model_name,
             }
         )
         return context
 
 
-class ListingResultsView(BaseListingView):
-    template_name = "wagtailimages/images/results.html"
-
-
 @permission_checker.require("change")
 def edit(request, image_id):
     Image = get_image_model()
@@ -202,7 +181,7 @@ def edit(request, image_id):
 
             messages.success(
                 request,
-                _("Image '{0}' updated.").format(image.title),
+                _("Image '%(image_title)s' updated.") % {"image_title": image.title},
                 buttons=[messages.button(edit_url, _("Edit again"))],
             )
             return redirect(redirect_url)
@@ -225,7 +204,8 @@ def edit(request, image_id):
                 request,
                 _(
                     "The source image file could not be found. Please change the source or delete the image."
-                ).format(image.title),
+                )
+                % {"image_title": image.title},
                 buttons=[
                     messages.button(
                         reverse("wagtailimages:delete", args=(image.id,)), _("Delete")
@@ -254,71 +234,86 @@ def edit(request, image_id):
     )
 
 
-def url_generator(request, image_id):
-    image = get_object_or_404(get_image_model(), id=image_id)
+class URLGeneratorView(generic.InspectView):
+    any_permission_required = ["change"]
+    model = get_image_model()
+    pk_url_kwarg = "image_id"
+    header_icon = "image"
+    page_title = "Generating URL"
+    template_name = "wagtailimages/images/url_generator.html"
 
-    if not permission_policy.user_has_permission_for_instance(
-        request.user, "change", image
-    ):
-        raise PermissionDenied
+    def get_page_subtitle(self):
+        return self.object.title
 
-    form = URLGeneratorForm(
-        initial={
-            "filter_method": "original",
-            "width": image.width,
-            "height": image.height,
-        }
-    )
+    def get_fields(self):
+        return []
 
-    return TemplateResponse(
-        request,
-        "wagtailimages/images/url_generator.html",
-        {
-            "image": image,
-            "form": form,
-        },
-    )
+    def get(self, request, image_id, *args, **kwargs):
+        self.object = get_object_or_404(self.model, id=image_id)
 
+        if not permission_policy.user_has_permission_for_instance(
+            request.user, "change", self.object
+        ):
+            raise PermissionDenied
 
-def generate_url(request, image_id, filter_spec):
-    # Get the image
-    Image = get_image_model()
-    try:
-        image = Image.objects.get(id=image_id)
-    except Image.DoesNotExist:
-        return JsonResponse({"error": "Cannot find image."}, status=404)
-
-    # Check if this user has edit permission on this image
-    if not permission_policy.user_has_permission_for_instance(
-        request.user, "change", image
-    ):
-        return JsonResponse(
-            {"error": "You do not have permission to generate a URL for this image."},
-            status=403,
+        self.form = URLGeneratorForm(
+            initial={
+                "filter_method": "original",
+                "width": self.object.width,
+                "height": self.object.height,
+            }
         )
 
-    # Parse the filter spec to make sure its valid
-    try:
-        Filter(spec=filter_spec).operations
-    except InvalidFilterSpecError:
-        return JsonResponse({"error": "Invalid filter spec."}, status=400)
+        return self.render_to_response(self.get_context_data())
 
-    # Generate url
-    signature = generate_signature(image_id, filter_spec)
-    url = reverse("wagtailimages_serve", args=(signature, image_id, filter_spec))
+    def get_context_data(self, **kwargs):
+        context = super().get_context_data(**kwargs)
+        context["form"] = self.form
+        return context
 
-    # Get site root url
-    try:
-        site_root_url = Site.objects.get(is_default_site=True).root_url
-    except Site.DoesNotExist:
-        site_root_url = Site.objects.first().root_url
 
-    # Generate preview url
-    preview_url = reverse("wagtailimages:preview", args=(image_id, filter_spec))
+class GenerateURLView(View):
+    def get(self, request, image_id, filter_spec):
+        # Get the image
+        Image = get_image_model()
+        try:
+            image = Image.objects.get(id=image_id)
+        except Image.DoesNotExist:
+            return JsonResponse({"error": "Cannot find image."}, status=404)
 
-    return JsonResponse(
-        {"url": site_root_url + url, "preview_url": preview_url}, status=200
-    )
+        # Check if this user has edit permission on this image
+        if not permission_policy.user_has_permission_for_instance(
+            request.user, "change", image
+        ):
+            return JsonResponse(
+                {
+                    "error": "You do not have permission to generate a URL for this image."
+                },
+                status=403,
+            )
+
+        # Parse the filter spec to make sure it's valid
+        try:
+            Filter(spec=filter_spec).operations
+        except InvalidFilterSpecError:
+            return JsonResponse({"error": "Invalid filter spec."}, status=400)
+
+        # Generate url
+        signature = generate_signature(image_id, filter_spec)
+        url = reverse("wagtailimages_serve", args=(signature, image_id, filter_spec))
+
+        # Get site root url
+        try:
+            site_root_url = Site.objects.get(is_default_site=True).root_url
+        except Site.DoesNotExist:
+            site_root_url = Site.objects.first().root_url
+
+        # Generate preview url
+        preview_url = reverse("wagtailimages:preview", args=(image_id, filter_spec))
+
+        return JsonResponse(
+            {"url": site_root_url + url, "preview_url": preview_url}, status=200
+        )
 
 
 def preview(request, image_id, filter_spec):
@@ -335,30 +330,37 @@ def preview(request, image_id, filter_spec):
         )
 
 
-@permission_checker.require("delete")
-def delete(request, image_id):
-    image = get_object_or_404(get_image_model(), id=image_id)
+class DeleteView(generic.DeleteView):
+    model = get_image_model()
+    pk_url_kwarg = "image_id"
+    permission_policy = permission_policy
+    permission_required = "delete"
+    header_icon = "image"
+    template_name = "wagtailimages/images/confirm_delete.html"
+    usage_url_name = "wagtailimages:image_usage"
+    delete_url_name = "wagtailimages:delete"
+    index_url_name = "wagtailimages:index"
+    page_title = gettext_lazy("Delete image")
 
-    if not permission_policy.user_has_permission_for_instance(
-        request.user, "delete", image
-    ):
-        raise PermissionDenied
+    def user_has_permission(self, permission):
+        return self.permission_policy.user_has_permission_for_instance(
+            self.request.user, permission, self.object
+        )
 
-    next_url = get_valid_next_url_from_request(request)
+    @property
+    def confirmation_message(self):
+        # This message will only appear in the singular, but we specify a plural
+        # so it can share the translation string with confirm_bulk_delete.html
+        return ngettext(
+            "Are you sure you want to delete this image?",
+            "Are you sure you want to delete these images?",
+            1,
+        )
 
-    if request.method == "POST":
-        image.delete()
-        messages.success(request, _("Image '{0}' deleted.").format(image.title))
-        return redirect(next_url) if next_url else redirect("wagtailimages:index")
-
-    return TemplateResponse(
-        request,
-        "wagtailimages/images/confirm_delete.html",
-        {
-            "image": image,
-            "next": next_url,
-        },
-    )
+    def get_success_message(self):
+        return _("Image '%(image_title)s' deleted.") % {
+            "image_title": self.object.title
+        }
 
 
 @permission_checker.require("add")
@@ -374,7 +376,7 @@ def add(request):
 
             messages.success(
                 request,
-                _("Image '{0}' added.").format(image.title),
+                _("Image '%(image_title)s' added.") % {"image_title": image.title},
                 buttons=[
                     messages.button(
                         reverse("wagtailimages:edit", args=(image.id,)), _("Edit")
@@ -396,12 +398,18 @@ def add(request):
     )
 
 
-def usage(request, image_id):
-    image = get_object_or_404(get_image_model(), id=image_id)
+class UsageView(generic.UsageView):
+    model = get_image_model()
+    paginate_by = USAGE_PAGE_SIZE
+    pk_url_kwarg = "image_id"
+    permission_policy = permission_policy
+    permission_required = "change"
+    header_icon = "image"
 
-    paginator = Paginator(image.get_usage(), per_page=USAGE_PAGE_SIZE)
-    used_by = paginator.get_page(request.GET.get("p"))
+    def user_has_permission(self, permission):
+        return self.permission_policy.user_has_permission_for_instance(
+            self.request.user, permission, self.object
+        )
 
-    return TemplateResponse(
-        request, "wagtailimages/images/usage.html", {"image": image, "used_by": used_by}
-    )
+    def get_page_subtitle(self):
+        return self.object.title
