@@ -8,17 +8,26 @@ from collections import defaultdict
 
 from django.conf import settings
 from django.contrib.auth import get_user_model
+from django.contrib.auth.models import Permission
 from django.contrib.contenttypes.models import ContentType
 from django.core.exceptions import ValidationError
+from django.core.serializers.json import DjangoJSONEncoder
 from django.db import models
 from django.utils import timezone
 from django.utils.functional import cached_property
 from django.utils.translation import gettext_lazy as _
 
 from wagtail.log_actions import registry as log_action_registry
+from wagtail.users.utils import get_deleted_user_display_name
 
 
 class LogEntryQuerySet(models.QuerySet):
+    def get_actions(self):
+        """
+        Returns a set of actions used by at least one log entry in this QuerySet
+        """
+        return set(self.order_by().values_list("action", flat=True).distinct())
+
     def get_user_ids(self):
         """
         Returns a set of user IDs of users who have created at least one log entry in this QuerySet
@@ -60,12 +69,21 @@ class LogEntryQuerySet(models.QuerySet):
         for log_entry in log_entries:
             ids_by_content_type[log_entry.content_type_id].append(log_entry.object_id)
 
-        instances_by_id = (
-            {}
-        )  # lookup of (content_type_id, stringified_object_id) to instance
+        instances_by_id = {}  # lookup of (content_type_id, stringified_object_id) to instance
         for content_type_id, object_ids in ids_by_content_type.items():
-            model = ContentType.objects.get_for_id(content_type_id).model_class()
-            model_instances = model.objects.in_bulk(object_ids)
+            try:
+                content_type = ContentType.objects.get_for_id(content_type_id)
+                model = content_type.model_class()
+            except ContentType.DoesNotExist:
+                model = None
+
+            if model:
+                model_instances = model.objects.in_bulk(object_ids)
+            else:
+                # The model class for the logged instance no longer exists,
+                # so we have no instance to return. Return None instead.
+                model_instances = {object_id: None for object_id in object_ids}
+
             for object_id, instance in model_instances.items():
                 instances_by_id[(content_type_id, str(object_id))] = instance
 
@@ -117,7 +135,37 @@ class BaseLogEntryManager(models.Manager):
         )
 
     def viewable_by_user(self, user):
-        return self.all()
+        if user.is_superuser:
+            return self.all()
+
+        # This will be called multiple times per request, so we cache those ids once.
+        if not hasattr(user, "_allowed_content_type_ids"):
+            # 1) Only query those permissions, where log entries exist for their content
+            # types.
+            used_content_type_ids = self.values_list(
+                "content_type_id", flat=True
+            ).distinct()
+            permissions = Permission.objects.filter(
+                content_type_id__in=used_content_type_ids
+            )
+            # 2) If the user has at least one permission for a content type, we add its
+            # id to the allowed-set.
+            allowed_content_type_ids = set()
+            for permission in permissions:
+                if permission.content_type_id in allowed_content_type_ids:
+                    continue
+
+                content_type = ContentType.objects.get_for_id(
+                    permission.content_type_id
+                )
+                if user.has_perm(
+                    "%s.%s" % (content_type.app_label, permission.codename)
+                ):
+                    allowed_content_type_ids.add(permission.content_type_id)
+
+            user._allowed_content_type_ids = allowed_content_type_ids
+
+        return self.filter(content_type_id__in=user._allowed_content_type_ids)
 
     def get_for_model(self, model):
         # Return empty queryset if the given object is not valid.
@@ -150,7 +198,7 @@ class BaseLogEntry(models.Model):
     label = models.TextField()
 
     action = models.CharField(max_length=255, blank=True, db_index=True)
-    data = models.JSONField(blank=True, default=dict)
+    data = models.JSONField(blank=True, default=dict, encoder=DjangoJSONEncoder)
     timestamp = models.DateTimeField(verbose_name=_("timestamp (UTC)"), db_index=True)
     uuid = models.UUIDField(
         blank=True,
@@ -184,6 +232,8 @@ class BaseLogEntry(models.Model):
 
     objects = BaseLogEntryManager()
 
+    wagtail_reference_index_ignore = True
+
     class Meta:
         abstract = True
         verbose_name = _("log entry")
@@ -198,9 +248,10 @@ class BaseLogEntry(models.Model):
         if not log_action_registry.action_exists(self.action):
             raise ValidationError(
                 {
-                    "action": _("The log action '{}' has not been registered.").format(
-                        self.action
+                    "action": _(
+                        "The log action '%(action_name)s' has not been registered."
                     )
+                    % {"action_name": self.action}
                 }
             )
 
@@ -221,8 +272,7 @@ class BaseLogEntry(models.Model):
         if self.user_id:
             user = self.user
             if user is None:
-                # User has been deleted. Using a string placeholder as the user id could be non-numeric
-                return _("user %(id)s (deleted)") % {"id": self.user_id}
+                return get_deleted_user_display_name(self.user_id)
 
             try:
                 full_name = user.get_full_name().strip()
